@@ -1,20 +1,33 @@
+/* Definitions for Windows process invocation.
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+This file is part of GNU Make.
+
+GNU Make is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 3 of the License, or (at your option) any later
+version.
+
+GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program.  If not, see <http://www.gnu.org/licenses/>.  */
+
 #ifndef SUB_PROC_H
 #define SUB_PROC_H
 
 /*
- * Component Name: 
+ * Component Name:
  *
- * $Date: 1997/08/27 20:34:23 $
+ * $Date: 2009/10/25 18:56:46 $
  *
- * $Source: /home/cvs/make/w32/include/sub_proc.h,v $
+ * $Source: /sources/make/make/w32/include/sub_proc.h,v $
  *
- * $Revision: 1.4 $
+ * $Id: sub_proc.h,v 1.11 2009/10/25 18:56:46 psmith Exp $
  */
 
-/* $Id: sub_proc.h,v 1.4 1997/08/27 20:34:23 psmith Exp $ */
-
-#ifdef WINDOWS32
-
 #define EXTERN_DECL(entry, args) extern entry args
 #define VOID_DECL void
 
@@ -23,7 +36,7 @@ EXTERN_DECL(HANDLE process_init_fd, (HANDLE stdinh, HANDLE stdouth,
 	HANDLE stderrh));
 EXTERN_DECL(long process_begin, (HANDLE proc, char **argv, char **envp,
 	char *exec_path, char *as_user));
-EXTERN_DECL(long process_pipe_io, (HANDLE proc, char *stdin_data, 
+EXTERN_DECL(long process_pipe_io, (HANDLE proc, char *stdin_data,
 	int stdin_data_len));
 EXTERN_DECL(long process_file_io, (HANDLE proc));
 EXTERN_DECL(void process_cleanup, (HANDLE proc));
@@ -31,6 +44,7 @@ EXTERN_DECL(HANDLE process_wait_for_any, (VOID_DECL));
 EXTERN_DECL(void process_register, (HANDLE proc));
 EXTERN_DECL(HANDLE process_easy, (char** argv, char** env));
 EXTERN_DECL(BOOL process_kill, (HANDLE proc, int signal));
+EXTERN_DECL(int process_used_slots, (VOID_DECL));
 
 /* support routines */
 EXTERN_DECL(long process_errno, (HANDLE proc));
@@ -44,4 +58,3 @@ EXTERN_DECL(int process_errcnt, (HANDLE proc));
 EXTERN_DECL(void process_pipes, (HANDLE proc, int pipes[3]));
 
 #endif
-#endif
